@@ -121,9 +121,9 @@ static uint32 DisAddr = 0x0000;
 static int NeedDisAddrChange = 0;
 
 static bool NeedPCBPToggle = 0;
-static int NeedStep = 0;
+static volatile int NeedStep = 0;
 static volatile int NeedRun = 0;
-
+static bool NeedBreak = 0;
 static bool InSteppingMode = 0;
 
 static void DrawRegs(RegType *rec, SDL_Surface *surface, uint32 *pixels, int highlight)
@@ -166,7 +166,8 @@ typedef enum
  PokeMeHL,
  ReadBPS,
  WriteBPS,
- DumpMem
+ DumpMem,
+ DumpGfxMem,
 } PromptType;
 
 static PromptType InPrompt = None;
@@ -191,24 +192,24 @@ static void DrawPrompt(SDL_Surface *surface, const SDL_Rect *rect)
  uint32 pitch32 = surface->pitch >> 2;
 
  pixels += ((rect->h / 2) - (4 * 7 / 2)) * pitch32;
- pixels += ((rect->w / 2) - (5 * 30 / 2));
+ pixels += ((rect->w / 2) - (5 * 39 / 2));
 
  for(unsigned int y = 0; y < 4 * 7; y++)
  {
   uint32 *row = pixels + y * pitch32;
-  for(unsigned int x = 0; x < 30 * 5; x++)
+  for(unsigned int x = 0; x < 39 * 5; x++)
    row[x] = MK_COLOR_A(0, 0, 0, 0xFF);
  }
 
- DrawTextTrans(pixels, surface->pitch, 30 * 5, (UTF8 *)"******************************", MK_COLOR_A(0xFF,0xFF,0xFF,0xFF), 0, 1);
- DrawTextTrans(pixels + 7 * pitch32, surface->pitch, 30 * 5, (UTF8 *)"*                            *", MK_COLOR_A(0xFF,0xFF,0xFF,0xFF), 0, 1);
- DrawTextTrans(pixels + 14 * pitch32, surface->pitch, 30 * 5, (UTF8 *)"*                            *", MK_COLOR_A(0xFF,0xFF,0xFF,0xFF), 0, 1);
- DrawTextTrans(pixels + 21 * pitch32, surface->pitch, 30 * 5, (UTF8 *)"******************************", MK_COLOR_A(0xFF,0xFF,0xFF,0xFF), 0, 1);
+ DrawTextTrans(pixels, surface->pitch, 39 * 5, (UTF8 *)"**************************************", MK_COLOR_A(0xFF,0xFF,0xFF,0xFF), 0, 1);
+ DrawTextTrans(pixels + 7 * pitch32, surface->pitch, 39 * 5, (UTF8 *)"*                                    *", MK_COLOR_A(0xFF,0xFF,0xFF,0xFF), 0, 1);
+ DrawTextTrans(pixels + 14 * pitch32, surface->pitch, 39 * 5, (UTF8 *)"*                                    *", MK_COLOR_A(0xFF,0xFF,0xFF,0xFF), 0, 1);
+ DrawTextTrans(pixels + 21 * pitch32, surface->pitch, 39 * 5, (UTF8 *)"**************************************", MK_COLOR_A(0xFF,0xFF,0xFF,0xFF), 0, 1);
 
- DrawTextTrans(pixels + 7 * pitch32 + 2 * 5, surface->pitch, (30 - 4) * 5, (UTF8 *)PromptText.c_str(), MK_COLOR_A(0xFF, 0x00, 0xFF, 0xFF), 0, 1);
- DrawTextTrans(pixels + 14 * pitch32 + 2 * 5, surface->pitch, (30 - 4) * 5, (UTF8 *)PromptAnswer.c_str(), MK_COLOR_A(0x00, 0xFF, 0x00, 0xFF), 0, 1);
+ DrawTextTrans(pixels + 7 * pitch32 + 2 * 5, surface->pitch, (39 - 4) * 5, (UTF8 *)PromptText.c_str(), MK_COLOR_A(0xFF, 0x00, 0xFF, 0xFF), 0, 1);
+ DrawTextTrans(pixels + 14 * pitch32 + 2 * 5, surface->pitch, (39 - 4) * 5, (UTF8 *)PromptAnswer.c_str(), MK_COLOR_A(0x00, 0xFF, 0x00, 0xFF), 0, 1);
 
- if(SDL_GetTicks() & 0x80)
+ if((SDL_GetTicks() & 0x80) && kb_cursor_pos < (39 - 4))
   DrawTextTrans(pixels + 14 * pitch32 + 2 * 5 + kb_cursor_pos * 5, surface->pitch, 5, (UTF8*)"▉", MK_COLOR_A(0x00, 0xFF, 0x00, 0xFF), 0, 1);
 }
 
@@ -381,19 +382,18 @@ void Debugger_Draw(SDL_Surface *surface, const SDL_Rect *rect)
  if(InPrompt)
   DrawPrompt(surface, rect);
 
- if(NeedStep == 3)
-  NeedStep--;
  LockDebuggerMutex(0);
 }
 
 // Function called from game thread
 static void CPUCallback(uint32 PC)
 {
- if(NeedStep == 2 && !InSteppingMode)
+ if((NeedStep == 2 && !InSteppingMode) || NeedBreak)
  {
   DisAddr = PC;
   NeedStep = 0;
   InSteppingMode = 1;
+  NeedBreak = 0;
  }
 
  if(NeedStep == 1)
@@ -402,7 +402,7 @@ static void CPUCallback(uint32 PC)
   NeedStep = 0;
  }
 
- while(InSteppingMode)
+ while(InSteppingMode && GameThreadRun)
  {
   DebuggerFudge();
   if(NeedStep == 2)
@@ -423,10 +423,17 @@ static void CPUCallback(uint32 PC)
 // Function called from game thread
 static void BPCallback(uint32 PC)
 {
- NeedStep = 2;
+ NeedBreak = 1;
  IsActive = 1;
 }
 
+// Function called from game thread, input driver code.
+void Debugger_ForceStepIfStepping(void)
+{
+ if(InSteppingMode)
+  NeedStep = 2;
+}
+
 // Call this function from any thread:
 bool Debugger_IsActive(void)
 {
@@ -597,6 +604,34 @@ int Debugger_Event(const SDL_Event *event)
 		    }
 		   }
 		  }
+                  else if(InPrompt == DumpGfxMem)
+                  {
+                   uint32 A1, A2;
+                   char fname[64];
+                   char *meow_str = tmp_c_str;
+                   bool logical = 1;
+
+                   if(meow_str[0] == '*')
+                   {
+                    meow_str++;
+                    logical = 0;
+                   }
+
+                   if(sscanf(meow_str, "%08x %08x %63[^\r\n]", &A1, &A2, fname) == 3)
+                   {
+                    FILE *fp = fopen(fname, "wb");
+                    if(fp)
+                    {
+                     LockDebuggerMutex(1);
+                     for(uint64 A = A1; A <= A2; A++)
+                     {
+                      fputc(CurGame->Debugger->GfxMemPeek(A, 1, 1), fp);
+                     }
+                     LockDebuggerMutex(0);
+                     fclose(fp);
+                    }
+                   }
+                  }
                   else if(InPrompt == EditRegs0 || InPrompt == EditRegs1 || InPrompt == EditRegs2)
                   {
                    uint32 RegValue = 0;
@@ -756,9 +791,17 @@ int Debugger_Event(const SDL_Event *event)
 		NeedPCBPToggle = 1;
 		break;
 	 case SDLK_s:
-		NeedStep = 3;
+		LockDebuggerMutex(1);
+		NeedStep = 2;
+		LockDebuggerMutex(0);
 		break;
 	 case SDLK_d:
+		if(event->key.keysym.mod & KMOD_SHIFT)
+		{
+		 InPrompt = DumpGfxMem;
+		 PromptText = "Dump Gfx Memory(start end filename)";
+		}
+		else
 		{
 		 InPrompt = DumpMem;
 		 PromptText = "Dump Memory(start end filename)";
