@@ -25,7 +25,7 @@
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
-
+#include <trio/trio.h>
 #include <locale.h>
 
 #ifdef WIN32
@@ -37,6 +37,7 @@
 #include "input.h"
 #include "joystick.h"
 #include "video.h"
+#include "opengl.h"
 #include "sound.h"
 #ifdef NETWORK
 #include "netplay.h"
@@ -60,6 +61,10 @@ static const char *CSD_stretch = "Stretch to fill screen.";
 static const char *CSD_videoip = "Enable bilinear interpolation.";
 static const char *CSD_special = "Enable specified special video scaler.";
 
+#ifdef MDFN_WANT_OPENGL_SHADERS
+static const char *CSD_pixshader = "Enable specified OpenGL pixel shader.";
+#endif
+
 static MDFNSetting DriverSettings[] =
 {
   #ifdef NETWORK
@@ -82,6 +87,9 @@ static MDFNSetting DriverSettings[] =
   { "nes.stretch", CSD_stretch, MDFNST_BOOL, "0" },
   { "nes.videoip", CSD_videoip, MDFNST_BOOL, "0" },
   { "nes.special", CSD_special, MDFNST_STRING, "none", NULL, NULL, MDFND_ValidateSpecialScalerSetting },
+  #ifdef MDFN_WANT_OPENGL_SHADERS
+  { "nes.pixshader", CSD_pixshader, MDFNST_STRING, "none", NULL, NULL },
+  #endif
 
   { "gb.xres", CSD_xres, MDFNST_UINT, "800" },
   { "gb.yres", CSD_yres, MDFNST_UINT, "600" },
@@ -93,6 +101,9 @@ static MDFNSetting DriverSettings[] =
   { "gb.stretch", CSD_stretch, MDFNST_BOOL, "0" },
   { "gb.videoip", CSD_videoip, MDFNST_BOOL, "0" },
   { "gb.special", CSD_special, MDFNST_STRING, "none", NULL, NULL, MDFND_ValidateSpecialScalerSetting },
+  #ifdef MDFN_WANT_OPENGL_SHADERS
+  { "gb.pixshader", CSD_pixshader, MDFNST_STRING, "none", NULL, NULL },
+  #endif
 
   { "gba.xres", CSD_xres, MDFNST_UINT, "800" },
   { "gba.yres", CSD_yres, MDFNST_UINT, "600" },
@@ -104,6 +115,9 @@ static MDFNSetting DriverSettings[] =
   { "gba.stretch", CSD_stretch, MDFNST_BOOL, "0" },
   { "gba.videoip", CSD_videoip, MDFNST_BOOL, "0" },
   { "gba.special", CSD_special, MDFNST_STRING, "none", NULL, NULL, MDFND_ValidateSpecialScalerSetting },
+  #ifdef MDFN_WANT_OPENGL_SHADERS
+  { "gba.pixshader", CSD_pixshader, MDFNST_STRING, "none", NULL, NULL },
+  #endif
 
   { "lynx.xres", CSD_xres, MDFNST_UINT, "800" },
   { "lynx.yres", CSD_yres, MDFNST_UINT, "600" },
@@ -115,6 +129,23 @@ static MDFNSetting DriverSettings[] =
   { "lynx.stretch", CSD_stretch, MDFNST_BOOL, "0" },
   { "lynx.videoip", CSD_videoip, MDFNST_BOOL, "0" },
   { "lynx.special", CSD_special, MDFNST_STRING, "none", NULL, NULL, MDFND_ValidateSpecialScalerSetting },
+  #ifdef MDFN_WANT_OPENGL_SHADERS
+  { "lynx.pixshader", CSD_pixshader, MDFNST_STRING, "none", NULL, NULL },
+  #endif
+
+  { "ngp.xres", CSD_xres, MDFNST_UINT, "640" },
+  { "ngp.yres", CSD_yres, MDFNST_UINT, "480" },
+  { "ngp.xscale", CSD_xscale, MDFNST_FLOAT, "4" },
+  { "ngp.yscale", CSD_yscale, MDFNST_FLOAT, "4" },
+  { "ngp.xscalefs", CSD_xscalefs, MDFNST_FLOAT, "3" },
+  { "ngp.yscalefs", CSD_yscalefs, MDFNST_FLOAT, "3" },
+  { "ngp.scanlines", CSD_scanlines, MDFNST_UINT, "0" },
+  { "ngp.stretch", CSD_stretch, MDFNST_BOOL, "0" },
+  { "ngp.videoip", CSD_videoip, MDFNST_BOOL, "0" },
+  { "ngp.special", CSD_special, MDFNST_STRING, "none", NULL, NULL, MDFND_ValidateSpecialScalerSetting },
+  #ifdef MDFN_WANT_OPENGL_SHADERS
+  { "ngp.pixshader", CSD_pixshader, MDFNST_STRING, "none", NULL, NULL },
+  #endif
 
   { "pce.xres", CSD_xres, MDFNST_UINT, "1024" },
   { "pce.yres", CSD_yres, MDFNST_UINT, "768" },
@@ -126,6 +157,9 @@ static MDFNSetting DriverSettings[] =
   { "pce.stretch", CSD_stretch, MDFNST_BOOL, "0" },
   { "pce.videoip", CSD_videoip, MDFNST_BOOL, "1" },
   { "pce.special", CSD_special, MDFNST_STRING, "none", NULL, NULL, MDFND_ValidateSpecialScalerSetting },
+  #ifdef MDFN_WANT_OPENGL_SHADERS
+  { "pce.pixshader", CSD_pixshader, MDFNST_STRING, "none", NULL, NULL },
+  #endif
 
   { "pcfx.xres", CSD_xres, MDFNST_UINT, "1024" },
   { "pcfx.yres", CSD_yres, MDFNST_UINT, "768" },
@@ -137,14 +171,20 @@ static MDFNSetting DriverSettings[] =
   { "pcfx.stretch", CSD_stretch, MDFNST_BOOL, "0" },
   { "pcfx.videoip", CSD_videoip, MDFNST_BOOL, "1" },
   { "pcfx.special", CSD_special, MDFNST_STRING, "none", NULL, NULL, MDFND_ValidateSpecialScalerSetting },
+  #ifdef MDFN_WANT_OPENGL_SHADERS
+  { "pcfx.pixshader", CSD_pixshader, MDFNST_STRING, "none", NULL, NULL },
+  #endif
 
   { "fs", "Enable fullscreen mode.", MDFNST_BOOL, "0", },
   { "vdriver", "Select video driver.  0 = OpenGL, 1 = SDL surface blits", MDFNST_UINT, "0", "0", "1" },
   { "doublebuf", "Enable video double buffering.", MDFNST_BOOL, "1" },
+
   { "ffspeed", "Fast-forwarding speed multiplier.", MDFNST_UINT, "4", "1", "15" },
   { "fftoggle", "Treat the fast-forward button as a toggle.", MDFNST_BOOL, "0" },
   { "ffnosound", "Silence sound output when fast-forwarding.", MDFNST_BOOL, "0" },
   { "autofirefreq", "Auto-fire frequency.", MDFNST_UINT, "3", "0", "1000" },
+  { "analogthreshold", "Threshold for detecting a \"button\" press on analog axis, in percent.", MDFNST_FLOAT, "75", "0", "100" },
+  { "ckdelay", "The length of time, in milliseconds, that a button/key corresponding to a \"dangerous\" command like power, reset, exit, etc. must be pressed before the command is executed.", MDFNST_UINT, "0", "0", "99999" },
   { "nes.fcexp", "Select Famicom expansion-port device.", MDFNST_STRING, "none", NULL, NULL, MDFND_ValidateFamicomExpansionPort },
   { "nes.input1", "Select input device for input port 1.", MDFNST_STRING, "gamepad", NULL, NULL, MDFND_ValidateNESInputPort },
   { "nes.input2", "Select input device for input port 2.", MDFNST_STRING, "gamepad", NULL, NULL , MDFND_ValidateNESInputPort },
@@ -241,7 +281,7 @@ static CFGSTRUCT fceuconfig[]={
 	ENDCFGSTRUCT
 };
 
-static void SaveConfig(void)
+static void SaveInputMapConfig(void)
 {	
         std::string tdir;
 
@@ -252,7 +292,7 @@ static void SaveConfig(void)
         SaveMDFNConfig(tdir.c_str(), fceuconfig);
 }
 
-static void LoadConfig(void)
+static void LoadInputMapConfig(void)
 {
 	std::string tdir;
 
@@ -267,66 +307,110 @@ static void LoadConfig(void)
 static void CreateDirs(void)
 {
  char *subs[6]={"mcs","mcm","snaps","gameinfo","sav","cheats"};
- char tdir[2048];
+ char *tdir;
  int x;
 
- #ifdef WIN32
- mkdir(DrBaseDirectory);
- for(x=0;x<6;x++)
+ MDFN_mkdir(DrBaseDirectory, S_IRWXU);
+ for(x = 0;x < 6;x++)
  {
-  sprintf(tdir,"%s"PSS"%s",DrBaseDirectory,subs[x]);
-  mkdir(tdir);
+  tdir = trio_aprintf("%s"PSS"%s",DrBaseDirectory,subs[x]);
+  MDFN_mkdir(tdir, S_IRWXU);
+  free(tdir);
  }
- #else
- mkdir(DrBaseDirectory,S_IRWXU);
- for(x=0;x<6;x++)
- {
-  sprintf(tdir,"%s"PSS"%s",DrBaseDirectory,subs[x]);
-  mkdir(tdir,S_IRWXU);
- }
- #endif
 }
 
-#ifndef WIN32
+#ifdef HAVE_SIGNAL
+
+typedef struct
+{
+ int number;
+ const char *message;
+} SignalInfo;
+
+static SignalInfo SignalDefs[] =
+{
+ #ifdef SIGINT
+ { SIGINT, "How DARE you interrupt me!\n" },
+ #endif
+
+ #ifdef SIGTERM
+ { SIGTERM, "MUST TERMINATE ALL HUMANS\n" },
+ #endif
+
+ #ifdef SIGHUP
+ { SIGHUP, "Reach out and hang-up on someone.\n" },
+ #endif
+
+ #ifdef SIGSEGV
+ { SIGSEGV, "Iyeeeeeeeee!!!  A segmentation fault has occurred.  Have a fluffy day.\n" },
+ #endif
+
+ #ifdef SIGPIPE
+ { SIGPIPE, "The pipe has broken!  Better watch out for floods...\n" },
+ #endif
+
+ #ifdef SIGBUS
+ /* SIGBUS can == SIGSEGV on some platforms, so put it after SIGSEGV */
+ { SIGBUS, "I told you to be nice to the driver.\n" },
+ #endif
+
+ #ifdef SIGFPE
+ { SIGFPE, "Those darn floating points.  Ne'er know when they'll bite!\n" },
+ #endif
+
+ #ifdef SIGALRM
+ { SIGALRM, "Don't throw your clock at the meowing cats!\n" },
+ #endif
+
+ #ifdef SIGABRT
+ { SIGABRT, "Abort, Retry, Ignore, Fail?\n" },
+ #endif
+ 
+ #ifdef SIGUSR1
+ { SIGUSR1, "Killing your processes is not nice.\n" },
+ #endif
+
+ #ifdef SIGUSR2
+ { SIGUSR2, "Killing your processes is not nice.\n" },
+ #endif
+};
+
 static void SetSignals(void (*t)(int))
 {
-  int sigs[10]={SIGINT,SIGTERM,SIGHUP,SIGPIPE,SIGSEGV,SIGFPE,SIGALRM,SIGABRT,SIGUSR1,SIGUSR2};
-  int x;
-  for(x=0;x<10;x++)
-   signal(sigs[x],t);
+  for(unsigned int x = 0; x < sizeof(SignalDefs) / sizeof(SignalInfo); x++)
+   signal(SignalDefs[x].number, t);
 }
 
 static void CloseStuff(int signum)
 {
-	//printf("%d\n",SDL_ThreadID());
+	//if(GameThread && SDL_ThreadID() == SDL_GetThreadID(GameThread))
+	//{
+	// printf("%d, %d %d %d\n",SDL_ThreadID(), MainThreadID, getpid(), getppid());
+	// kill(getpid(), signum);
+	// for(;;) SDL_Delay(10000);
+	//}
 	//DriverKill();
         printf(_("\nSignal %d has been caught and dealt with...\n"),signum);
-        switch(signum)
-        {
-         case SIGINT:printf(_("How DARE you interrupt me!\n"));break;
-         case SIGTERM:printf(_("MUST TERMINATE ALL HUMANS\n"));break;
-         case SIGHUP:printf(_("Reach out and hang-up on someone.\n"));break;
-         case SIGPIPE:printf(_("The pipe has broken!  Better watch out for floods...\n"));break;
-         case SIGSEGV:printf(_("Iyeeeeeeeee!!!  A segmentation fault has occurred.  Have a fluffy day.\n"));break;
-	 /* So much SIGBUS evil. */
-	 #ifdef SIGBUS
-	 #if(SIGBUS!=SIGSEGV)
-         case SIGBUS:printf(_("I told you to be nice to the driver.\n"));break;
-	 #endif
-	 #endif
-         case SIGFPE:printf(_("Those darn floating points.  Ne'er know when they'll bite!\n"));break;
-         case SIGALRM:printf(_("Don't throw your clock at the meowing cats!\n"));break;
-         case SIGABRT:printf(_("Abort, Retry, Ignore, Fail?\n"));break;
-         case SIGUSR1:
-         case SIGUSR2:printf(_("Killing your processes is not nice.\n"));break;
-        }
+
+	for(unsigned int x = 0; x < sizeof(SignalDefs) / sizeof(SignalInfo); x++)
+	 if(SignalDefs[x].number == signum)
+	 {
+	  printf("%s", _(SignalDefs[x].message));
+	  break;
+	 }
+
+        if(GameThread && SDL_ThreadID() == MainThreadID)
+	{
+	 SDL_KillThread(GameThread);
+	 GameThread = NULL;
+	}
         exit(1);
 }
 #endif
 
 static ARGPSTRUCT *MDFN_Internal_Args = NULL;
 
-static int HokeyPokeyFallDown(char *name, char *value)
+static int HokeyPokeyFallDown(const char *name, const char *value)
 {
  if(!MDFNI_SetSetting(name + 1, value))
   return(0);
@@ -419,7 +503,7 @@ static int DoArgs(int argc, char *argv[], char **filename)
 
 static volatile int NeedVideoChange = 0;
 int GameLoop(void *arg);
-static volatile int GameThreadRun = 0;
+volatile int GameThreadRun = 0;
 void MDFND_Update(uint32 *XBuf, int16 *Buffer, int Count);
 
 bool sound_active;	// true if sound is enabled and initialized
@@ -465,6 +549,7 @@ int LoadGame(const char *path)
 
 	if(netconnect)
 	 MDFND_NetworkConnect();
+
 	GameThreadRun = 1;
 	GameThread = SDL_CreateThread(GameLoop, NULL);
 
@@ -777,7 +862,7 @@ void SetJoyReadMode(int mode)	// 0 for events, 1 for manual function calling to
 
  /* Only block if we're calling this from within the game loop(it is also called from within LoadGame(), called in the main loop). */
  if(SDL_ThreadID() != MainThreadID) //if(GameThread && (SDL_ThreadID() != SDL_GetThreadID(GameThread))) - oops, race condition with the setting of GameThread, possibly...
-  while(JoyModeChange)
+  while(JoyModeChange && GameThreadRun)
    SDL_Delay(1);
 }
 
@@ -793,27 +878,28 @@ void RefreshThrottleFPS(int multiplier)
         CurGameSpeed = multiplier;
 }
 
-
 extern uint32 uppow2(uint32);
+int sdlhaveogl = 0;
+
 int main(int argc, char *argv[])
 {
 	int ret;
 	char *needie = NULL;
+
 	DrBaseDirectory=GetBaseDirectory();
 
 	#ifdef ENABLE_NLS
 	setlocale(LC_ALL, "");
+
 	#ifdef WIN32
-	puts(DrBaseDirectory);
         bindtextdomain(PACKAGE, DrBaseDirectory);
-        bind_textdomain_codeset(PACKAGE, "UTF-8");
-        textdomain(PACKAGE);
 	#else
 	bindtextdomain(PACKAGE, LOCALEDIR);
+	#endif
+
 	bind_textdomain_codeset(PACKAGE, "UTF-8");
 	textdomain(PACKAGE);
 	#endif
-	#endif
 
 	if(SDL_Init(SDL_INIT_VIDEO)) /* SDL_INIT_VIDEO Needed for (joystick config) event processing? */
 	{
@@ -822,23 +908,30 @@ int main(int argc, char *argv[])
 	 return(-1);
 	}
 
+	// SDL_GL_LoadLibrary returns 0 on success, -1 on failure
+	if(SDL_GL_LoadLibrary(NULL) == 0)
+         sdlhaveogl = 1;
+	else
+	 sdlhaveogl = 0;
+
 	MainThreadID = SDL_ThreadID();
         if(!(ret=MDFNI_Initialize(DrBaseDirectory, DriverSettings)))
          return(-1);
 
         SDL_EnableUNICODE(1);
 
-        #ifndef WIN32
+        #ifdef HAVE_SIGNAL
         SetSignals(CloseStuff);
         #endif
 
 	CreateDirs();
 	MakeInputConfigStruct();
 	MakeMednafenArgsStruct();
-        LoadConfig();
+        LoadInputMapConfig();
 
 	if(!DoArgs(argc,argv, &needie))
 	{
+	 SaveInputMapConfig();
 	 MDFNI_Kill();
 	 DeleteInternalArgs();
 	 return(-1);
@@ -869,8 +962,8 @@ int main(int argc, char *argv[])
 	 // None of our systems have a height about 256, but we really should
 	 // add hints in the MDFNGameInfo struct about the max dimensions of the screen
 	 // buffer.
-         VTBuffer[0] = (uint32 *)malloc(CurGame->pitch * 256 * sizeof(uint32));
-         VTBuffer[1] = (uint32 *)malloc(CurGame->pitch * 256 * sizeof(uint32));
+         VTBuffer[0] = (uint32 *)malloc(CurGame->pitch * 256);
+         VTBuffer[1] = (uint32 *)malloc(CurGame->pitch * 256);
 	 VTLineWidths[0] = (MDFN_Rect *)calloc(256, sizeof(MDFN_Rect));
 	 VTLineWidths[1] = (MDFN_Rect *)calloc(256, sizeof(MDFN_Rect));
 	 VTOSDBuffer[0] = (uint32 *)calloc(512 * 240, sizeof(uint32));
@@ -954,11 +1047,12 @@ int main(int argc, char *argv[])
 	 free(VTBuffer[1]);
 	 VTBuffer[1] = NULL;
 	}
-	#ifndef WIN32
+
+	#ifdef HAVE_SIGNAL
 	SetSignals(SIG_IGN);
 	#endif
 
-	SaveConfig();
+	SaveInputMapConfig();
         MDFNI_Kill();
 
 	SDL_Quit();
@@ -993,7 +1087,7 @@ static int ThrottleCheckFS(void)
 
  if((ttime - ltime) < (tfreq / desiredfps ))
  {
-  if(!sound_active && !NoWaiting && !nothrottle)
+  if(!sound_active && !NoWaiting && !nothrottle && GameThreadRun)
   {
    int64 delay;
    delay=(tfreq/desiredfps)-(ttime-ltime);
@@ -1103,7 +1197,7 @@ void MDFND_Update(uint32 *XBuf, int16 *Buffer, int Count)
   if((last_btime + 100) < SDL_GetTicks())
   {
    //puts("Eep");
-   while(VTReady) SDL_Delay(1);
+   while(VTReady && GameThreadRun) SDL_Delay(1);
   }
   if(!VTReady)
   {
