@@ -44,6 +44,7 @@ typedef struct
         int stretch;
         int special;
         int scanlines;
+	std::string pixshader;
 } CommonVS;
 
 static CommonVS _video;
@@ -186,17 +187,6 @@ int KillVideo(int subsys)
  return(0);
 }
 
-#ifdef WIN32
-#include <windows.h>
-
-BOOL CALLBACK choochoo(HWND hwnd, LPARAM lParam)
-{
- //SetWindowLong(hwnd, GWL_STYLE, WS_OVERLAPPED|WS_CAPTION|WS_SYSMENU|WS_MINIMIZEBOX);
- SetWindowPos(hwnd, NULL, 50, 50, 0, 0, SWP_NOSIZE); // | SWP_FRAMECHANGED);
-}
-
-#endif
-
 static void GenerateRects(MDFN_Rect &DisplayRect)
 {
  src_rect.x = DisplayRect.x;
@@ -324,18 +314,10 @@ int InitVideo(MDFNGI *gi)
 
  _video.special = GetSpecialScalerID(special_string);
 
- #ifdef WIN32
- int needwmfix = 0;
- /* The window tends to get trashed on fullscreen->windowed transitions on Win32, so implement this workaround. */
- if(screen && (cur_flags & SDL_FULLSCREEN) && !_fullscreen)
- {
-  needwmfix = 1;
-  //extern HWND SDL_Window;
-  //SDL_QuitSubSystem(SDL_INIT_VIDEO);
-  //SDL_InitSubSystem(SDL_INIT_VIDEO);
-  //SDL_SetVideoMode(256, 240, 8, 0);
-  //screen = NULL;
- }
+ #ifdef MDFN_WANT_OPENGL_SHADERS
+ _video.pixshader = MDFN_GetSettingS(std::string(std::string(gi->shortname) + "." + std::string("pixshader")).c_str());
+ #else
+ _video.pixshader = "none";
  #endif
 
  CurrentScaler = _video.special ? &Scalers[_video.special - 1] : NULL;
@@ -369,8 +351,12 @@ int InitVideo(MDFNGI *gi)
  }
 
  if(vdriver == 0)
-  flags |= SDL_OPENGL;
-
+ {
+  if(sdlhaveogl)
+   flags |= SDL_OPENGL;
+  else
+   MDFN_PrintError(_("Could not load OpenGL library, disabling OpenGL usage!"));
+ }
  exres = _video.xres;
  eyres = _video.yres;
  exs = _fullscreen ? _video.xscalefs : _video.xscale;
@@ -449,6 +435,14 @@ int InitVideo(MDFNGI *gi)
   return(0);
  }
  MDFN_printf(_("OpenGL: %s\n"), (cur_flags & SDL_OPENGL) ? _("Yes") : _("No"));
+
+ if(cur_flags & SDL_OPENGL)
+ {
+  MDFN_indent(1);
+  MDFN_printf(_("Pixel shader: %s\n"), _video.pixshader.c_str());
+  MDFN_indent(-1);
+ }
+
  MDFN_printf(_("Fullscreen: %s\n"), _fullscreen ? _("Yes") : _("No"));
  MDFN_printf(_("Special Scaler: %s\n"), _video.special ? Scalers[_video.special - 1].name : _("None"));
  if(!_video.scanlines)
@@ -481,7 +475,7 @@ int InitVideo(MDFNGI *gi)
 
  if(cur_flags & SDL_OPENGL)
  {
-  if(!InitOpenGL(evideoip, _video.scanlines, screen))
+  if(!InitOpenGL(evideoip, _video.scanlines, _video.pixshader, screen))
   {
    KillVideo(1);
    MDFN_indent(-1);
@@ -498,13 +492,6 @@ int InitVideo(MDFNGI *gi)
  MDFN_indent(-1);
  SDL_ShowCursor(0);
 
- #ifdef WIN32
- if(needwmfix && !(cur_flags & SDL_FULLSCREEN))
- {
-  EnumThreadWindows(GetCurrentThreadId(),choochoo, NULL);
- }
- #endif
-
  int rs, gs, bs, as;
 
  if(cur_flags & SDL_OPENGL)
@@ -562,62 +549,8 @@ int InitVideo(MDFNGI *gi)
  return 1;
 }
 
-void BlitScreen(uint32 *XBuf, MDFN_Rect *DisplayRect, MDFN_Rect *LineWidths, uint32 *OSDBuffer)
+static void SubBlit(void)
 {
- if(!screen) return;
-
- GenerateRects(*DisplayRect);
- source_surface->pixels = XBuf;
- OSDSurface->pixels = OSDBuffer;
-
- if(OSDBuffer)
- {
-  OSDRect.w = 256;
-  if(Netplay_GetTextView() || IsConsoleCheatConfigActive() || Debugger_IsActive())
-   OSDRect.w = 512;
-
-  FPS_Draw(OSDSurface, &OSDRect);
-
-  if(Debugger_IsActive())
-   Debugger_Draw(OSDSurface, &OSDRect);
-  else if(Netplay_GetTextView())
-   DrawNetplayTextBuffer(OSDSurface, &OSDRect);
-  else if(IsConsoleCheatConfigActive())
-   DrawCheatConsole(OSDSurface, &OSDRect);
- }
-
- int y;
- int last_y = src_rect.y;
- int last_x = LineWidths[src_rect.y].x;
- int last_width = LineWidths[src_rect.y].w;
- SDL_Rect src_rect_save;
- SDL_Rect dest_rect_save;
- bool SkipMultiWidths = 0;
-
- if(LineWidths[0].w == ~0) // Skip multi line widths code?
-  SkipMultiWidths = 1;
-
- if(SkipMultiWidths)
-  goto SkipIntoLoop; // Yes, this is horribly ugly bad, and it generates tons of compiler warnings but I'm...uhm...sleepy?
-
- for(y = src_rect.y; y < (src_rect.y + src_rect.h + 1); y++)
- {
-  if(y == (src_rect.y + src_rect.h) || LineWidths[y].x != last_x || LineWidths[y].w != last_width)
-  {
-   src_rect_save = src_rect;
-   dest_rect_save = dest_rect;
-   src_rect.x = last_x;
-   src_rect.w = last_width;
-   src_rect.y = last_y;
-   src_rect.h = y - last_y;
-
-   dest_rect.y = dest_rect_save.y + (last_y - src_rect_save.y) * dest_rect.h / src_rect_save.h;
-   dest_rect.h = src_rect.h * dest_rect.h / src_rect_save.h;
-   //printf("%d %d\n", dest_rect.y, dest_rect.h);
-   //printf("%d %d\n", y, last_y);
-
-   SkipIntoLoop:
-
    if(CurrentScaler)
    {
     uint8 *screen_pixies;
@@ -709,18 +642,70 @@ void BlitScreen(uint32 *XBuf, MDFN_Rect *DisplayRect, MDFN_Rect *LineWidths, uin
     else
      BlitNonGL(source_surface, &src_rect, &src_rect, &dest_rect, screen, _video.scanlines);
    }
+}
 
-   if(SkipMultiWidths)
-    goto SkipOutOfLoop;
+void BlitScreen(uint32 *XBuf, MDFN_Rect *DisplayRect, MDFN_Rect *LineWidths, uint32 *OSDBuffer)
+{
+ if(!screen) return;
 
-   src_rect = src_rect_save;
-   dest_rect = dest_rect_save;
-   last_y = y;
-   last_width = LineWidths[y].w;
-   last_x =  LineWidths[y].x;
+ GenerateRects(*DisplayRect);
+ source_surface->pixels = XBuf;
+ OSDSurface->pixels = OSDBuffer;
+
+ if(OSDBuffer)
+ {
+  OSDRect.w = 256;
+  if(Netplay_GetTextView() || IsConsoleCheatConfigActive() || Debugger_IsActive())
+   OSDRect.w = 512;
+
+  FPS_Draw(OSDSurface, &OSDRect);
+
+  if(Debugger_IsActive())
+   Debugger_Draw(OSDSurface, &OSDRect);
+  else if(Netplay_GetTextView())
+   DrawNetplayTextBuffer(OSDSurface, &OSDRect);
+  else if(IsConsoleCheatConfigActive())
+   DrawCheatConsole(OSDSurface, &OSDRect);
+ }
+
+ if(LineWidths[0].w == ~0) // Skip multi line widths code?
+ {
+  SubBlit();
+ }
+ else
+ {
+  int y;
+  int last_y = src_rect.y;
+  int last_x = LineWidths[src_rect.y].x;
+  int last_width = LineWidths[src_rect.y].w;
+  SDL_Rect src_rect_save;
+  SDL_Rect dest_rect_save;
+
+  for(y = src_rect.y; y < (src_rect.y + src_rect.h + 1); y++)
+  {
+   if(y == (src_rect.y + src_rect.h) || LineWidths[y].x != last_x || LineWidths[y].w != last_width)
+   {
+    src_rect_save = src_rect;
+    dest_rect_save = dest_rect;
+    src_rect.x = last_x;
+    src_rect.w = last_width;
+    src_rect.y = last_y;
+    src_rect.h = y - last_y;
+
+    dest_rect.y = dest_rect_save.y + (last_y - src_rect_save.y) * dest_rect.h / src_rect_save.h;
+    dest_rect.h = src_rect.h * dest_rect.h / src_rect_save.h;
+
+    // Blit here!
+    SubBlit();
+
+    src_rect = src_rect_save;
+    dest_rect = dest_rect_save;
+    last_y = y;
+    last_width = LineWidths[y].w;
+    last_x =  LineWidths[y].x;
+   }
   }
  }
- SkipOutOfLoop:
 
  if(OSDBuffer)
  {
