@@ -1057,11 +1057,11 @@ void MDFNI_CheatSearchEnd(int type, uint64 v1, uint64 v2, unsigned int bytelen,
          doexclude = 1;
         break;
       case 4: // Value decreased
-        if(!(ramval < ccval))
+        if(ramval >= ccval)
          doexclude = 1;
         break;
       case 5: // Value increased
-        if(!(ramval < ccval))
+        if(ramval <= ccval)
          doexclude = 1;
         break;
      }
