@@ -4,6 +4,7 @@
  *  Copyright (C) 1998 BERO
  *  Copyright (C) 2003 Xodnizel
  *  Mapper 12 code Copyright (C) 2003 CaH4e3
+ *  UNL6035052 code Copyright (C) 2005 CaH4e3
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -544,11 +545,10 @@ int Mapper49_Init(CartInfo *info)
 
 static void M115PW(uint32 A, uint8 V)
 {
- setprg8(A,V);
  if(EXPREGS[0]&0x80)
- {
-  setprg16(0x8000,EXPREGS[0]&7);
- }
+  setprg32(0x8000,(EXPREGS[0]&7) >> 1);
+ else
+  setprg8(A,V);
 }
 
 static void M115CW(uint32 A, uint8 V)
@@ -574,7 +574,7 @@ static void M115Power(CartInfo *info)
 
 int Mapper115_Init(CartInfo *info)
 {
- if(!(GenMMC3_Init(info, 512, 256, 0, 0)))
+ if(!(GenMMC3_Init(info, 512, 512, 0, 0)))
   return(0);
  cwrap=M115CW;
  pwrap=M115PW;
@@ -1079,6 +1079,39 @@ int Mapper148_Init(CartInfo *info)
  return(1);
 }
 
+
+static uint8 UNL6035052_lut[4] = { 0x00, 0x02, 0x02, 0x03 };
+
+static DECLFW(UNL6035052ProtWrite)
+{
+  EXPREGS[0]=UNL6035052_lut[V&3];
+}
+
+static DECLFR(UNL6035052ProtRead)
+{
+  return EXPREGS[0];
+}
+
+static void UNL6035052Power(CartInfo *info)
+{
+  GenMMC3Power(info);
+  SetWriteHandler(0x4020,0x7FFF,UNL6035052ProtWrite);
+  SetReadHandler(0x4020,0x7FFF,UNL6035052ProtRead);
+}
+
+int UNL6035052_Init(CartInfo *info)
+{
+  if(!GenMMC3_Init(info, 128, 256, 0, 0))
+   return(0);
+
+  info->Power=UNL6035052Power;
+  EXPRCount = 1;
+
+  return(1);
+}
+
+
+
 static void TKSPPU(uint32 A)
 {
  //static uint8 z;
