@@ -21,6 +21,7 @@
 /* None of this code should use any of the iNES bank switching wrappers. */
 
 #include "mapinc.h"
+#include "../nsf.h"
 
 static void (*sfun)(int P);
 static void (*psfun)(void);
@@ -522,13 +523,16 @@ static void Do5PCMHQ()
    MMC5Sound.BC[2]=SOUNDTS;
 } 
 
+void MMC5HiSync(int32 ts)
+{
+ int x;
+ for(x=0;x<3;x++) MMC5Sound.BC[x]=ts;
+}
 
 static DECLFW(Mapper5_SW)
 {
  A&=0x1F;
 
- GameExpSound.HiFill=MMC5RunSoundHQ;
-
  switch(A)
  {
   case 0x10:if(psfun) psfun();MMC5Sound.rawcontrol=V;break;
@@ -604,15 +608,8 @@ void MMC5RunSoundHQ(void)
   Do5PCMHQ();
 }
 
-void MMC5HiSync(int32 ts)
-{
- int x;
- for(x=0;x<3;x++) MMC5Sound.BC[x]=ts;
-}
-
 static void Mapper5_ESI(EXPSOUND *ep)
 {
- ep->RChange=Mapper5_ESI;
  if(FSettings.SndRate)
  {
   sfun=Do5SQHQ;
@@ -625,21 +622,24 @@ static void Mapper5_ESI(EXPSOUND *ep)
  }
  memset(MMC5Sound.BC,0,sizeof(MMC5Sound.BC));
  memset(MMC5Sound.vcount,0,sizeof(MMC5Sound.vcount));
- ep->HiSync=MMC5HiSync;
+
+ ep->HiSync = MMC5HiSync;
+ ep->HiFill = MMC5RunSoundHQ;
+ ep->RChange = Mapper5_ESI;
 }
 
-int NSFMMC5_Init(EXPSOUND *ep)
+int NSFMMC5_Init(EXPSOUND *ep, bool MultiChip)
 {
   memset(&MMC5Sound,0,sizeof(MMC5Sound));
   mul[0]=mul[1]=0;
   if(!(ExRAM=(uint8*)malloc(1024)))
    return(0);
   Mapper5_ESI(ep);
-  SetWriteHandler(0x5c00,0x5fef,MMC5_ExRAMWr);
+  NSFECSetWriteHandler(0x5c00,0x5fef,MMC5_ExRAMWr);
   SetReadHandler(0x5c00,0x5fef,MMC5_ExRAMRd); 
   MMC5HackCHRMode=2;
-  SetWriteHandler(0x5000,0x5015,Mapper5_SW);
-  SetWriteHandler(0x5205,0x5206,Mapper5_write);
+  NSFECSetWriteHandler(0x5000,0x5015,Mapper5_SW);
+  NSFECSetWriteHandler(0x5205,0x5206,Mapper5_write);
   SetReadHandler(0x5205,0x5206,MMC5_read);
   return(1);
 }
@@ -785,7 +785,9 @@ static int GenMMC5_Init(CartInfo *info, int wsize, int battery)
  MMC5HackVROMPTR=CHRptr[0];
  MMC5HackCHRMode=0;
  MMC5HackSPMode=MMC5HackSPScroll=MMC5HackSPPage=0;
- Mapper5_ESI(&GameExpSound);
+
+
+ Mapper5_ESI(&info->CartExpSound);
 
  return(1);
 }
