@@ -28,6 +28,7 @@
 #include "sound.h"
 #include "memory.h"
 #include "cart.h"
+#include "nsf.h"
 
 /*	TODO:  Add code to put a delay in between the time a disk is inserted
 	and the when it can be successfully read/written to.  This should
@@ -412,47 +413,27 @@ static void HQSync(int32 ts)
  FBC=ts;
 }
 
-/*
-static DECLFR(FDSBIOSPatch)
-{
- if(FDSRegs[5]&0x4)
- {
-  X.X=FDSRead4031(0x4031);
-  FDSWrite(0x4024,X.A);
-  X.A=X.X;
-  return(0x60);
- }
- else
- {
-  return(0x58);
-  //puts("Write");
- }
-}
-*/
-
 static void FDS_ESI(EXPSOUND *ep)
 {
  if(FSettings.SndRate)
- {
   fdso.cycles=(int64)1<<39;
- }
-//  fdso.cycles=(int64)32768*FDSClock/(FSettings.SndRate *16);
- SetReadHandler(0x4040,0x407f,FDSWaveRead);
- SetWriteHandler(0x4040,0x407f,FDSWaveWrite);
- SetWriteHandler(0x4080,0x408A,FDSSWrite);
- SetReadHandler(0x4090,0x4092,FDSSRead);
-
- //SetReadHandler(0xE7A3,0xE7A3,FDSBIOSPatch);
 }
 
-int NSFFDS_Init(EXPSOUND *ep)
+int NSFFDS_Init(EXPSOUND *ep, bool MultiChip)
 {
  memset(&fdso,0,sizeof(fdso));
+
  FDS_ESI(ep);
+
  ep->HiSync=HQSync;
  ep->HiFill=RenderSoundHQ;
  ep->RChange=FDS_ESI;
 
+ SetReadHandler(0x4040,0x407f,FDSWaveRead);
+ NSFECSetWriteHandler(0x4040,0x407f,FDSWaveWrite);
+ NSFECSetWriteHandler(0x4080,0x408A,FDSSWrite);
+ SetReadHandler(0x4090,0x4092,FDSSRead);
+
  return(1);
 }
 
@@ -954,10 +935,21 @@ static void FDSInit(void)
 
  memset(&fdso,0,sizeof(fdso));
 
- FDS_ESI(&GameExpSound);
- GameExpSound.HiSync=HQSync;
- GameExpSound.HiFill=RenderSoundHQ;
- GameExpSound.RChange=FDS_ESI;
+ EXPSOUND TmpExpSound;
+ memset(&TmpExpSound, 0, sizeof(TmpExpSound));
+
+ SetReadHandler(0x4040,0x407f,FDSWaveRead);
+ SetWriteHandler(0x4040,0x407f,FDSWaveWrite);
+ SetWriteHandler(0x4080,0x408A,FDSSWrite);
+ SetReadHandler(0x4090,0x4092,FDSSRead);
+
+ FDS_ESI(&TmpExpSound);
+
+ TmpExpSound.HiSync=HQSync;
+ TmpExpSound.HiFill=RenderSoundHQ;
+ TmpExpSound.RChange=FDS_ESI;
+
+ GameExpSound.push_back(TmpExpSound);
 
  InDisk=0;
  SelectDisk=0;
