@@ -614,6 +614,7 @@ int iNESLoad(const char *name, MDFNFILE *fp, NESGameType *gt)
 	 iNESFree();
 	 return(0);
 	}
+
 	MDFN_LoadGameSave(&iNESCart);
 
 	gt->Power = iNES_Power;
@@ -621,6 +622,9 @@ int iNESLoad(const char *name, MDFNFILE *fp, NESGameType *gt)
 	gt->Close = iNES_Close;
 	gt->StateAction = iNES_StateAction;
 
+	if(iNESCart.CartExpSound.HiFill)
+	 GameExpSound.push_back(iNESCart.CartExpSound);
+
 	if(head.ROM_type & 4)
 	{
          SetReadHandler(0x7000,0x71FF,TrainerRead);
