@@ -1159,6 +1159,7 @@ int MDFNPPU_Loop(int skip)
                                    of this delay. 
                                 */
    X6502_Run(12);
+
    if(MDFNGameInfo->nes.type==GIT_NSF)
     DoNSFFrame();
    else
@@ -1180,7 +1181,6 @@ int MDFNPPU_Loop(int skip)
    {
     for(scanline=0;scanline<240;)       //scanline is incremented in  DoLine.  Evil. :/
      DoLine(skip);
-
     if(MMC5Hack && (ScreenON || SpriteON)) MMC5_hb(scanline);
    }
   } /* else... to if(ppudead) */
@@ -1394,3 +1394,30 @@ void MDFNPPU_Init(void)
  RedoRL();
 }
 
+
+// If hl is true, do not cause any change in the underlying hardware state.
+uint32 NESPPU_GfxMemPeek(uint32 A, unsigned int bsize, bool hl)
+{
+ A &= 0x3FFF;
+ uint32 ret = 0;
+
+ for(int n = 0; n < bsize; n++)
+ {
+  if(A < 0x2000)
+   ret |= VPage[A >> 10][A] << (n * 8);
+  else if(A >= 0x3f00)
+   ret |= PALRAM[A & ((A & 3)? 0x1F : 0x0C)];
+  else
+   ret |= vnapage[(A >> 10) & 0x3][A & 0x3FF];
+
+  A = (A + 1) & 0x3FFF;
+ }
+ return(ret);
+}
+
+ // If hl is true, write to the underlying ROM or RAM directly, if present
+void NESPPU_GfxMemPoke(uint32 A, uint32 V, unsigned int bsize, bool hl)
+{
+
+}
+
