@@ -22,3 +22,6 @@ int MDFNPPU_StateAction(StateMem *sm, int load, int data_only);
 bool MDFNNES_ToggleLayer(int which);
 uint32 NESPPU_GetRegister(const std::string &name);
 void NESPPU_SetRegister(const std::string &name, uint32 value);
+uint32 NESPPU_GfxMemPeek(uint32 A, unsigned int bsize, bool hl);
+void NESPPU_GfxMemPoke(uint32 A, uint32 V, unsigned int bsize, bool hl);
+
