@@ -1,3 +1,6 @@
+#ifndef __NES_SOUND_H
+#define __NES_SOUND_H
+
 #include "sound-driver.h"
 
 typedef struct __EXPSOUND {
@@ -8,7 +11,9 @@ typedef struct __EXPSOUND {
 	   void (*Kill)(void);
 } EXPSOUND;
 
-extern EXPSOUND GameExpSound;
+#include <vector>
+
+extern std::vector<EXPSOUND> GameExpSound;
 
 int GetSoundBuffer(float **W);
 int FlushEmulateSound(int);
@@ -32,3 +37,4 @@ void MDFNNES_SetSoundVolume(uint32 volume);
 void MDFNNES_SetSoundMultiplier(double multiplier);
 void MDFNNES_Sound(int Rate);
 
+#endif
