@@ -37,6 +37,9 @@ static uint32   HalfCount;
 static uint8 LastCmd;
 static uint32 SampleFreq;
 
+static bool ADPCMLP;
+static uint32 LSampleFreq; // For ADPCM lowpass filter
+
 static uint8 ReadBuffer;
 static int32 ReadPending;
 static int32 WritePending;
@@ -206,6 +209,9 @@ bool ADPCM_Init()
 	ad_ref_index = 0;
 
 	ADPCMSynth.volume(0.375f); //0.50f); //0.375f); //0.50f); //0.300f);
+	LSampleFreq = 0xFFFFFFFF;
+	ADPCMLP = MDFN_GetSettingB("pce.adpcmlp");
+
 	return TRUE;
 }
 
@@ -309,7 +315,7 @@ uint8 ADPCM_ReadBuffer()
 void ADPCM_WriteBuffer(uint8            data)
 {
 	//printf("%d ", SampleFreq);
-        WritePending = 76 + -3.428571428571428354f * (SampleFreq + 1); // Icky kludge for Sherlock Holmes and Record of Lodoss War
+        WritePending = (int32)(76 + -3.428571428571428354f * (SampleFreq + 1)); // Icky kludge for Sherlock Holmes and Record of Lodoss War
         WritePendingValue = data;
 }
 
@@ -385,7 +391,7 @@ static void ADPCM_PB_Run(int32 basetime, int32 run_time)
  // Watch out for overflows, which are currently prevented
  // by calling ADPCM_Run() every VDC scanline.
 
- int32 bigdivacc = (double)1789772.727272 * 4 * 65536 / 32000 * pce_overclocked;
+ int32 bigdivacc = (int32)((double)1789772.727272 * 4 * 65536 / 32087.5 * pce_overclocked);
 
  //printf("%d\n", run_time);
  bigdiv -= run_time * 65536;
@@ -421,7 +427,7 @@ static void ADPCM_PB_Run(int32 basetime, int32 run_time)
     pcm = (pcm * CurrentVolume) >> 8;
     extern int8 CDDAStatus;
     //if(CDDAStatus == 1) pcm >>= 1;
-    if(CDDAStatus == 1) pcm *= 0.83;
+    if(CDDAStatus == 1) pcm = (int32)(pcm * 0.83);
     uint32 synthtime = ((basetime + (bigdiv >> 16)));
     if(pce_overclocked > 1) 
      synthtime /= pce_overclocked;
@@ -460,6 +466,23 @@ void ADPCM_Update(void)
  int32 run_time = timestamp - lastts;
  int32 meow;
 
+ if(ADPCMLP && SampleFreq != LSampleFreq)
+ {
+  if(SampleFreq >= 14)
+  {
+   int rolloff = (int)((((double)32087.5 / (16 - SampleFreq)) / 2) * 0.70);
+
+   ADPCMSynth.treble_eq( blip_eq_t::blip_eq_t(-1000, rolloff, FSettings.SndRate));
+  }
+  else
+  {
+   int rolloff = (int)((((double)32087.5 / (16 - SampleFreq)) / 2) * 0.80);
+
+   ADPCMSynth.treble_eq( blip_eq_t::blip_eq_t(-1000, rolloff, FSettings.SndRate));
+  }
+  LSampleFreq = SampleFreq;
+ }
+
  while(run_time > 0)
  {
   meow = min(WritePending?WritePending:run_time, ReadPending?ReadPending:run_time, CDROM_GetNextEventTime(), run_time);
@@ -512,6 +535,7 @@ void ADPCM_ResetTS(void)
 
 void ADPCM_SetFreq(uint32 freq)
 {
+	//printf("%d\n", freq);
 	SampleFreq = freq;
 }
 
@@ -531,7 +555,7 @@ void ADPCM_FadeOut(int32 ms)
 	}
 	else if (CurrentVolume > 0)
 	{
-		FadeCycle = ((7159090.0 / ((double)CurrentVolume / (double)VolumeStep)) * (double)ms) / 1000.0;
+		FadeCycle = (int32)(((7159090.0 / ((double)CurrentVolume / (double)VolumeStep)) * (double)ms) / 1000.0);
 		bFadeOut	= TRUE;
 		bFadeIn	= FALSE;
 	}
@@ -554,7 +578,7 @@ void ADPCM_FadeIn(int32	ms)
 	}
 	else if (InitialVolume - CurrentVolume > 0)
 	{
-		FadeCycle = ((7159090.0 / (((double)InitialVolume - (double)CurrentVolume) / (double)VolumeStep)) * (double)ms) / 1000.0;
+		FadeCycle = (int32)(((7159090.0 / (((double)InitialVolume - (double)CurrentVolume) / (double)VolumeStep)) * (double)ms) / 1000.0);
 		bFadeOut = FALSE;
 		bFadeIn  = TRUE;
 	}
@@ -572,9 +596,9 @@ static uint32 ADPCM_AdvanceClock(int32	cycles)
 	{
 		ClockCount += cycles;
 
-		while (ClockCount >= FadeCycle)
+		while (ClockCount >= FadeCycle * pce_overclocked)
 		{
-			ClockCount -= FadeCycle;
+			ClockCount -= FadeCycle * pce_overclocked;
 
 			if (bFadeOut)
 			{
