@@ -31,8 +31,8 @@
 #define BCD(A)	((A / 10) * 16 + (A % 10))		// convert INT --> BCD
 #define INT(B)	((B / 16) * 10 + (B % 16))		// convert BCD --> INT
 
-#define CD_DATA_TRANSFER_RATE	126000
-//126000
+static int32 CD_DATA_TRANSFER_RATE;
+
 static bool8	bBRAMEnabled;
 static uint8	_Port[15];
 static uint32	SectorAddr, SectorCount;
@@ -95,9 +95,9 @@ static INLINE void CDFADER_AdvanceClock(int32 cycles) // 21477270 / 3 = 7159090
         {
                 FadeClockCount += cycles;
 
-                while (FadeClockCount >= FadeCycle)
+                while (FadeClockCount >= FadeCycle * pce_overclocked)
                 {
-                        FadeClockCount -= FadeCycle;
+                        FadeClockCount -= FadeCycle * pce_overclocked;
 
                         if (bFadeOut)
                         {
@@ -419,6 +419,8 @@ int32 CDROM_Init()
 	if (!CDIF_Init())
 		return -1;
 
+	CD_DATA_TRANSFER_RATE = 126000 * MDFN_GetSettingUI("pce.cdspeed");
+
         CurrentCdVolume = InitialCdVolume = 65536;
         VolumeStep      = InitialCdVolume / 100;
 
@@ -1280,6 +1282,9 @@ uint32 CDROM_AdvanceClock(int32 basetime, int32	clock)
 
 	  if(FSettings.SndRate)
 	  {
+	   if(pce_overclocked > 1)
+	    synthtime /= pce_overclocked;
+
            CDDASynth.offset(synthtime, sample[0] - last_sample[0], &sbuf[0]);
 	   CDDASynth.offset(synthtime, sample[1] - last_sample[1], &sbuf[1]);
 
@@ -1385,6 +1390,10 @@ int CDROM_StateAction(StateMem *sm, int load, int data_only)
 	 SFEND
 	};
 
+        // Call StateAction here so that ReadBufferIndex and ReadByteCount are loaded with the
+	// correct values for the function below(when loading).
+        int ret = MDFNSS_StateAction(sm, load, data_only, StateRegs, "CDRM");
+
 	SFORMAT ArrayStateRegs[] =
 	{
 	 SFARRAY16(CDDASectorBuffer, 1176),
@@ -1393,8 +1402,9 @@ int CDROM_StateAction(StateMem *sm, int load, int data_only)
 	 SFARRAY(CmdArgBuffer, sizeof(CmdArgBuffer)),
 	 SFEND
 	};
-	int ret = MDFNSS_StateAction(sm, load, data_only, StateRegs, "CDRM");
+
 	ret &= MDFNSS_StateAction(sm, load, data_only, ArrayStateRegs, "CDAR");
+
 	if(load)
 	{
          if((ReadBufferIndex + ReadByteCount) > sizeof(ReadBuffer))
