@@ -41,9 +41,10 @@ void (*HuC6280_Run)(int32 cycles) = NULL;
 #define _tcount          HuCPU.tcount
 #define _IRQlow          HuCPU.IRQlow
 
+#define REDOSPEEDCACHE() { HuCPU.speed_shift_cache = ((HuCPU.speed ^ 1) << 1); }
 #define ADDCYC(x)       \
 {       \
- int __x=x << ((HuCPU.speed ^ 1) << 1);     \
+ int __x=x << HuCPU.speed_shift_cache;	\
  _tcount+=__x;  \
  _count-=__x;        \
  HuCPU.timestamp+=__x;        \
@@ -251,8 +252,8 @@ static uint8 ZNTable[256];
 				_A = HuCPU.MPR[i];	\
 		}	
 
-#define CSL	HuCPU.speed = 0
-#define CSH	HuCPU.speed = 1
+#define CSL	HuCPU.speed = 0; REDOSPEEDCACHE()
+#define CSH	HuCPU.speed = 1; REDOSPEEDCACHE()
 
 #define RMB(bitto)	x &= ~(1 << (bitto & 7))
 #define SMB(bitto)	x |= 1 << (bitto & 7)
@@ -464,6 +465,9 @@ void HuC6280_Reset(void)
  HuCPU.timer_value = 0;
  HuCPU.timer_status = 0;
  HuCPU.in_block_move = 0;
+ HuCPU.cpoint = 0;
+ HuCPU.preexec = 0;
+
  _IRQlow=MDFN_IQRESET;
 }
   
@@ -483,7 +487,8 @@ void HuC6280_Init(void)
 void HuC6280_Power(void)
 {
  _count=_tcount=_IRQlow=_PC=_A=_X=_Y=_S=_P=_PI=0;
- HuCPU.timestamp=0;
+
+ HuCPU.timestamp = 0;
 
  for(int i = 0; i < 9; i++)
  {
@@ -504,7 +509,9 @@ static void HuC6280_RunDebug(int32 cycles)
 
         _count+=cycles * pce_overclocked;
 
-        if(HuCPU.in_block_move && _count > 0)
+	PenguinPower:
+
+        if(HuCPU.in_block_move && _count > 0 && !HuCPU.cpoint)
         {
          switch(HuCPU.in_block_move)
          {
@@ -521,11 +528,13 @@ static void HuC6280_RunDebug(int32 cycles)
          int32 temp;
          static uint8 b1;
 
-         if(_IRQlow)
+         if(_IRQlow && !HuCPU.cpoint)
          {
           if(_IRQlow&MDFN_IQRESET)
           {
            HuCPU.speed = 0;
+	   REDOSPEEDCACHE();
+
            HuCPU.IRQMask = HuCPU.IRQMaskDelay = 7;
            HuC6280_SetMPR(7, 0);
            _PC=RdMem(0xFFFE);
@@ -573,7 +582,6 @@ static void HuC6280_RunDebug(int32 cycles)
          }
 
          _PC &= 0xFFFF;     // Our cpu core can only handle PC going about 8192 bytes over, so make sure it never gets that far...
-         _PI=_P;
 
 	 if(HuCPU.ReadHook || HuCPU.WriteHook)
 	 {
@@ -613,9 +621,13 @@ static void HuC6280_RunDebug(int32 cycles)
          #define RdOp  RdOpNorm
          #define WrMem WrMemNorm
 
+	 HuCPU.cpoint = 1;
          PCEDBG_TestFoundBPoint();
          if(HuCPU.CPUHook) HuCPU.CPUHook(_PC);
+	 if(!HuCPU.cpoint) goto PenguinPower;
+	 HuCPU.cpoint = 0;
 
+	 _PI=_P;
          HuCPU.IRQMaskDelay = HuCPU.IRQMask;
          b1=RdOp(_PC);
 
@@ -681,6 +693,15 @@ static void HuC6280_RunNormal(int32 cycles)
         #undef _PC
         #define _PC pbackus
 
+
+	// This handles the (rare) case of a save state being saved in step mode in the debugger
+	if(HuCPU.cpoint && _count > 0)
+	{
+	 HuCPU.cpoint = 0;
+	 goto SephirothBishie;
+	}
+
+
 	if(HuCPU.in_block_move && _count > 0)
 	{
 	 switch(HuCPU.in_block_move)
@@ -703,6 +724,8 @@ static void HuC6280_RunNormal(int32 cycles)
 	  if(_IRQlow&MDFN_IQRESET)
 	  {
 	   HuCPU.speed = 0;
+	   REDOSPEEDCACHE();
+
            HuCPU.IRQMask = HuCPU.IRQMaskDelay = 7;
 	   HuC6280_SetMPR(7, 0);
 	   _PC=RdMem(0xFFFE);
@@ -748,6 +771,8 @@ static void HuC6280_RunNormal(int32 cycles)
  	                                   /* major speed hit. */
 	 }
 
+	 SephirothBishie:
+
          _PC &= 0xFFFF;     // Our cpu core can only handle PC going about 8192 bytes over, so make sure it never gets that far...
 	 _PI=_P;
 	 HuCPU.IRQMaskDelay = HuCPU.IRQMask;
@@ -884,6 +909,7 @@ int HuC6280_StateAction(StateMem *sm, int load, int data_only)
   SFVARN(HuCPU.bmt_dest, "IBM_DEST"),
   SFVARN(HuCPU.bmt_length, "IBM_LENGTH"),
   SFVARN(HuCPU.bmt_alternate, "IBM_ALTERNATE"),
+  SFVARN(HuCPU.cpoint, "CPoint"),
   SFEND
  };
 
@@ -891,6 +917,12 @@ int HuC6280_StateAction(StateMem *sm, int load, int data_only)
  love.push_back(SSDescriptor(SFCPU, "CPU"));
  love.push_back(SSDescriptor(SFCPUC, "CPUC"));
 
+
+ if(load)
+ {
+  HuCPU.cpoint = 0; // For older save states.
+ }
+
  HuCPU.IRQMask ^= 7;
  HuCPU.IRQMaskDelay ^= 7;
  int ret = MDFNSS_StateAction(sm, load, data_only, love);
@@ -916,6 +948,7 @@ int HuC6280_StateAction(StateMem *sm, int load, int data_only)
    HuCPU.IRQlow = (HuCPU.IRQlow & (0x4 | 0x20)) | ((HuCPU.IRQlow & 1) << 1) | ((HuCPU.IRQlow & 2) >> 1);
   // Update MPR cache
   HuC6280_FlushMPRCache();
+  REDOSPEEDCACHE();
  }
  return(ret);
 }
