@@ -1,3 +1,5 @@
+#ifndef _PCE_VDC_H
+#define _PCE_VDC_H
 void VDC_SetPixelFormat(int rshift, int gshift, int bshift);
 void VDC_RunFrame(uint32 *pXBuf, MDFN_Rect *LineWidths, int skip);
 bool VDC_ToggleLayer(int which);
@@ -18,3 +20,7 @@ void VDC_SetRegister(std::string name, uint32 value);
 
 int VDC_StateAction(StateMem *sm, int load, int data_only);
 
+uint32 VDC_GfxMemPeek(uint32 A, unsigned int bsize, bool hl);
+void VDC_GfxMemPoke(uint32 A, uint32 V, unsigned int bsize, bool hl);
+
+#endif
