@@ -384,7 +384,7 @@ static int ReadStateChunk(StateMem *st, SFORMAT *sf, int size, int data_only)
  return 1;
 }
 
-int CurrentState=0;
+int CurrentState = 0;
 extern int geniestage;
 
 static void MakeStatePreview(uint8 *dest, uint32 *fb, MDFN_Rect *LineWidths, int xdiv, int ydiv)
@@ -404,17 +404,19 @@ static void MakeStatePreview(uint8 *dest, uint32 *fb, MDFN_Rect *LineWidths, int
    uint32 r,g,b;
 
    r=g=b=0;
+   x = 0;
+
    for(y=0;y<ydiv;y++)
-    for(x=0;x<xdiv;x++)
+    for(x = 0; x < xdiv; x++)
     {
      uint32 real_x;
      uint32 pixel; 
      int nr, ng, nb;
 
      if(LineWidths[0].w != ~0)
-      real_x = 256 * LineWidths[sqy * ydiv + y].x + 256 * (LineWidths[sqy * ydiv + y].w * sqx * xdiv + x) / MDFNGameInfo->width;
+      real_x = 256 * LineWidths[sqy * ydiv + y].x + 256 * (LineWidths[sqy * ydiv + y].w * (sqx * xdiv + x)) / MDFNGameInfo->width;
      else
-      real_x = 256 * MDFNGameInfo->DisplayRect.x + 256 * (MDFNGameInfo->DisplayRect.w * sqx * xdiv + x) / MDFNGameInfo->width;
+      real_x = 256 * MDFNGameInfo->DisplayRect.x + 256 * (MDFNGameInfo->DisplayRect.w * (sqx * xdiv + x)) / MDFNGameInfo->width;
 
      pixel = fb[(real_x >> 8) + ((sqy * ydiv) + y) * MDFNGameInfo->pitch / sizeof(uint32)];
      DECOMP_COLOR(pixel, nr, ng, nb);
@@ -426,9 +428,10 @@ static void MakeStatePreview(uint8 *dest, uint32 *fb, MDFN_Rect *LineWidths, int
      pixel = fb[((real_x >> 8) + 1) + ((sqy * ydiv) + y) * MDFNGameInfo->pitch / sizeof(uint32)];
      DECOMP_COLOR(pixel, nr, ng, nb);
 
-     r += nr * (real_x & 0xFF);
-     g += ng * (real_x & 0xFF);
-     b += nb * (real_x & 0xFF);     
+     r += nr * ((real_x & 0xFF));
+     g += ng * ((real_x & 0xFF));
+     b += nb * ((real_x & 0xFF));
+
     }
    *dest++ = r >> bitcow;
    *dest++ = g >> bitcow;
@@ -476,11 +479,12 @@ int MDFNSS_StateAction(StateMem *st, int load, int data_only, std::vector <SSDes
        return(0);
      }
     }
-    if(!found) // Not found.  We are sad!
-    {
-    }
     if(smem_seek(st, -total, SEEK_CUR) < 0)
      return(0);
+    if(!found && !section->optional) // Not found.  We are sad!
+    {
+	return(0);
+    }
    }
   }
  }
@@ -493,11 +497,11 @@ int MDFNSS_StateAction(StateMem *st, int load, int data_only, std::vector <SSDes
  return(1);
 }
 
-int MDFNSS_StateAction(StateMem *st, int load, int data_only, SFORMAT *sf, const char *name)
+int MDFNSS_StateAction(StateMem *st, int load, int data_only, SFORMAT *sf, const char *name, bool optional)
 {
  std::vector <SSDescriptor> love;
 
- love.push_back(SSDescriptor(sf, name));
+ love.push_back(SSDescriptor(sf, name, optional));
  return(MDFNSS_StateAction(st, load, data_only, love));
 }
 
@@ -757,10 +761,25 @@ void MDFNSS_CheckStates(void)
 void MDFNI_SelectState(int w)
 {
  gzFile fp;
- if(w == -1) { StateShow = 0; return; }
+
+ if(w == -1) 
+ {  
+  StateShow = 0; 
+  return; 
+ }
  MDFNI_SelectMovie(-1);
 
- CurrentState=w;
+ if(w == 666 + 1)
+  CurrentState = (CurrentState + 1) % 10;
+ else if(w == 666 - 1)
+ {
+  CurrentState--;
+
+  if(CurrentState < 0 || CurrentState > 9)
+   CurrentState = 9;
+ }
+ else
+  CurrentState = w;
  StateShow = MDFND_GetTime() + 2000;
 
  fp = gzopen(MDFN_MakeFName(MDFNMKF_STATE,CurrentState,NULL).c_str(),"rb");
