@@ -7,8 +7,25 @@
 #endif
 
 #include <assert.h>
-
 #include <inttypes.h>
+
+#if HAVE_MKDIR
+ #if MKDIR_TAKES_ONE_ARG
+  #define MDFN_mkdir(a, b) mkdir(a)
+ #else
+  #define MDFN_mkdir(a, b) mkdir(a, b)
+ #endif
+#else
+ #if HAVE__MKDIR
+  /* Plain Win32 */
+  #define MDFN_mkdir(a, b) _mkdir(a)
+ #else
+  #error "Don't know how to create a directory on this system."
+ #endif
+#endif
+
+
+
 typedef int8_t int8;
 typedef int16_t int16;
 typedef int32_t int32; 
@@ -80,4 +97,9 @@ typedef struct
 #define TRUE 1
 #endif
 
+#include <assert.h>
+
+#undef require
+#define require( expr ) assert( expr )
+
 #endif
